#!/usr/bin/perl

# 
# $Id: mysqlstatus.pl,v 1.1 2004/11/23 17:13:02 matt Exp $
#

=head1 NAME

mysqlstatus - outputs current status of mysql to a readable format for rrdtool

=head1 SYNOPSIS

Connects to a MySQL server and outputs data for a SNMP collector like rrdtool

=head1 DEPENDENCIES

  MySQL
  DBI
  DBD::mysql
  ~/.my.cnf with [mysql] configured

Configure .my.cnf the same as you would for use with the mysql client.

=cut

eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
    if 0; # not running under some shell
use strict;

use vars qw( $VERSION );

$VERSION = "1.4";

use Mysql::Replication::Mysql;   my $mysql   = new Mysql::Replication::Mysql;
use Mysql::Replication::Utility; my $utility = new Mysql::Replication::Utility;

use Getopt::Std;
use vars qw( $opt_r $opt_v);
getopts('rv');

CheckOption();

exit 1;

##
#  Subroutines
##

sub getStatus
{

=head1 getStatus

	Connect to the database and run the SHOW STATUS query. 

=cut

	my ($homedir)  = (getpwuid ($<))[7];
	my $dotfile    = "$homedir/.my.cnf";

	unless ( -e $dotfile ) {
		my @lines = "[mysql]\nuser=root\npass=\n";
		$utility->file_write($dotfile, @lines);
		chmod(0700, $dotfile);
	};

	my $dot = $mysql->parse_dot_file(".my.cnf", "[mysql]", 0);
	my ($dbh, $dsn, $drh) = $mysql->connect($dot, 1, 0);

	my $sth = $mysql->query( $dbh, "SHOW STATUS");

	my %result;

	for( my $j=0; $j<$sth->rows; $j++ )
	{
		my ($var_name, $value) = $sth->fetchrow_array;
		$result{$var_name} = $value;
	};

	$sth->finish(); 
	$dbh->disconnect(); 

	return \%result;
}

sub printServerStatus($$)
{
	my ($type, $status) = @_; 

	if( $type eq "queries")
	{
		if (defined($status)){ 
			print "mysql_queriessec:".$status->{Questions}.":mysql_connections:".$status->{Connections}.":mysql_slowqueries:".$status->{Slow_queries}."\n";
		}
		else{ 
			print "mysql_queriessec:0:mysql_connections:0:mysql_slowqueries:0\n";
		}	 
	}
	elsif( $type eq "type" )
	{
		if (defined($status)){ 
			print "mysql_select:".$status->{Com_select}	.":mysql_changedb:".$status->{Com_change_db}. 
				":mysql_update:".$status->{Com_update}.":mysql_insert:".$status->{Com_insert}. 
				":mysql_delete:".$status->{Com_delete}."\n";
		}
		else {
			print "mysql_select:0:mysql_changedb:0:mysql_update:0:mysql_insert:0:mysql_delete:0\n";
		}		
	}
} 

sub CheckOption()
{
	my $option = $ARGV[0];

	unless ( $option ) {
		die "\n Need an argument like this:\n\n $0 [ type | queries ]\n\n";
	}

	if ( $option eq "type") {
		my $status = getStatus();
		printServerStatus("type", $status);
	}
	elsif ( $option eq "queries"){
		my $status = getStatus();
		printServerStatus("queries", $status);
	}
	else
	{
		die "\n\n $0 [ type | queries ]\n\n";
	}
}

__END__;

=head1 CHANGELOG

 v1.4 - Matt
       Removed MATT::* dependencies, imported the new
         object methods from Mysql::Replication::*
       Also die with usage if invalid action is passed

 v1.3 - Matt
      Updated pod docs for pod2text README file
      Create ~/.my.cnf if it doesn't exist

 v1.2 - Matt 
      Use MATT::Mysql for connection ( gets MySQL user 
        info from .my.cnf instead of storing it in the 
        script file.)
      Removed a few redundant lines of code.

 v1.1 written by Rob Lensen

=head1 AUTHORS

 Rob Lensen (rob@bsdfreaks.nl)
 Matt Simerson (matt@tnpi.biz)

=head1 COPYRIGHT

Copyright (c) 2003-2004, Rob Lensen & The Network People, Inc.
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the The Network People, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  

=cut
